module G2R
  module RDBMS
    class Column
      attr_accessor :name

      def initialize(name)
        @name = RDBMS.transform_name(name)
        @primary_key = false
        @foreign_key = false
      end

      def ==(other_column)
        @name == other_column.name
      end

      # ========================================================================
      # PRIMARY KEY
      # ========================================================================
      # Define the column as a primary key
      def primary_key
        @primary_key = true
        self
      end

      # Checks if the column is a primary key
      def primary_key?
        @primary_key
      end

      # ========================================================================
      # FOREIGN KEY
      # ========================================================================
      # Define the column as a foreign key
      def foreign_key(target_table)
        @foreign_key = true
        @target_table = RDBMS.transform_name(target_table)
        self
      end

      # Check if the column is a foreign key
      def foreign_key?
        @foreign_key
      end

      # Table the foreign key is targeting
      def target_table
        @target_table
      end
    end
  end
end
